'''
Script for fitting the Q factor of a ringdown measurement

@Matthijs de Jong
2019-09-02 
'''
import numpy as np
import pandas as pd
from scipy.optimize import curve_fit
import matplotlib.pyplot as plt
import json

#Load any and all measurement parameters saved by the measurement script
measparam = json.load(open("Measurement parameters.json"))

#Include additional parameters.
parameters = {
        
        }

#Merge the two dictionaries (defined above and the imported one from the measurement scipt)
parameters = {**measparam, **parameters}


def linearcurve(x, a, b):
    return a*x+b
    
def fit(f, x, y, p0, pnames, verbose=False):
    """ Fitting function. It uses scipy's curve_fit with bounds. It outputs the optimal 
    fitting parameters popt, the covariances pcov, the errors on the optimized parameters
    perr, and the r squared r_squared to measure the goodness of the fit. 
    """
    # fit
    popt, pcov = curve_fit(f, x, y, p0=p0)

    # print fit parameters
    perr = np.sqrt(np.diag(pcov))
    if verbose: print("{:>10}{:>10}{:>10}{:>10}{:>10}".format("pnames", "p0", "pmin", "pmax", "popt"))

    for jdx in range(len(popt)):
        if verbose: print("{:>10}{:10.4f}{:10.4f} +/- {:.4f} ({:.4f}%)".format(
            pnames[jdx], p0[jdx], popt[jdx], perr[jdx], perr[jdx]/popt[jdx]*100))
    
    # calculate r^2 of the fit
    residuals = y - f(x, *popt)
    ss_res = np.sum(residuals**2)
    ss_tot = np.sum((y-np.mean(y))**2)
    r_squared = 1 - (ss_res / ss_tot)
    if verbose: print("r^2 = {:.2f}".format(r_squared))
    
    return popt, pcov, perr, r_squared

def linear_fit(xdata, ydata):
    a = -1
    b = -60
    
    pnames = ["a",     "b"]  # names of the parameters
    p0   =   [a,      b]  # initial fitting parameters
    # perform the fit
    popt, pcov, perr, r_squared = fit(linearcurve, xdata, ydata, p0, pnames)

    return popt, pcov, perr, r_squared   
    
Dataset = pd.read_csv("Ringdown.csv",header = 0)
Dataset.drop('Unnamed: 0', axis = 1, inplace = True)

columnum = np.linspace(0,int(Dataset.shape[1]/2)-1,int(Dataset.shape[1]/2))
columnum = columnum*2
for idx,val in enumerate(columnum):
    val = int(val)
    time0 = Dataset.iloc[:,val]
    power0 = Dataset.iloc[:,val+1]

    startcutoff = np.where(time0 > 10.0)[0][0]
    try:
        endcut = np.where(power0 < -95)[0][0]
    except IndexError:
        try:
            endcut = np.where(power0 < -75)[0][0]
        except IndexError:
            endcut = np.where(power0 < -55)[0][0]

    timecut = time0[startcutoff:endcut]
    powercut = power0[startcutoff:endcut]
    if timecut.shape[0] >= 10:
        popt, pcov, perr, r_squared = linear_fit(timecut, powercut)
    
        fittedQ = linearcurve(timecut, popt[0], popt[1])
        #print(popt[0]) #Decay in dBm per timestep
        #print(popt[1])
        decaytime = -popt[0]/(2*np.pi)
        #decaytime = np.log(10)*popt[0] #Half-time in timesteps
        #decaytimes = -decaytime*(timecut.iloc[10] - timecut.iloc[0])/10 #decay time in s
        '''
        if 0<= val <=parameters["Dithersteps"]:
            Peakfrequency = 172361
        elif 21<= val <= 42: 
            Peakfrequency = 266731
        elif 43 <= val <= 64:
            Peakfrequency = 266729
        '''    
        num = int(val/(2*parameters["Dithersteps"])-0.499)
        Peakfrequency = parameters["Peaklist"][idx]    
        print("Frequency: {:.0f} Hz".format(Peakfrequency))    
        Qfactor = Peakfrequency/decaytime
    
        #print(decaytime)
        print("Q-factor: {:.0f}".format(Qfactor))
    
        fig1 = plt.figure()
        ax1 = fig1.add_subplot(111)
    
        ax1.set_xlabel("Time (s)")
        ax1.set_ylabel("Power (dBm)")
    
        ax1.plot(Dataset.iloc[:,val],Dataset.iloc[:,val+1], label = 'data')
        ax1.plot(timecut, fittedQ, label = 'Fit')
    
        ax1.set_title('Q = {:.2f}, f = {:.3f} kHz'.format(Qfactor,Peakfrequency/1e3))
        ax1.legend()
    
        fig1.savefig('Fitted Q {}.png'.format(idx), bbox_inches = 'tight', dpi = 300)
        plt.close(fig1)