# -*- coding: utf-8 -*-
"""
Created on Mon Mar 21 16:12:02 2022

@author: matthijsdejong
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from scipy.integrate import odeint
from scipy.signal import stft
from scipy.interpolate import interp1d
import json

def doubleres(y, t, params):
    x1,v1,x2,v2 = y
    omega_1,gamma_1,omega_2, gamma_2,J = params
    derivs = [v1,-gamma_1*v1-omega_1**2*x1 - J**2*x2,v2,-gamma_2*v2-omega_2**2*x2-J**2*x1]
    return derivs

def Simdoubleres(timedata,J,x10,x20):
    #Print that it's working
    print("Running with J = {:.1f}, x10 = {:.3f}, x20 = {:.3f}".format(J,x10,x20))
    splittot = 8.6 #Total observed split between the modes, in Hz

    #Set some constants
    gamma_1 = 0.34*2 #Decay rate of resonator 1, in Hz (x2 bc FWHM)
    gamma_2 = 3*2
    
    #Calculate how much nu should be to get the splitting we see
    omega_1 = 240336.65
    nu = np.abs(np.real(splittot**2/2 + 1j*gamma_1/2 - 1j*gamma_2/2 - np.sqrt((gamma_1*1j + omega_1**2)*splittot**2 - J**4))/omega_1)
    #Calculate the split in frequencies based on the coupling J

    print('nu = {:.2f}'.format(nu))
    
    #Revise the frequencies.
    omega_1 = 240336.65+nu/2 #240341.5 #resonance frequency of resonator 1, in Hz
    omega_2 = 240336.65-nu/2 #240331.9 
    
    #J = 500#680#4e5 #3e3
    #tconst = -2.5

    #x10 = 0.08 #Initial displacement
    v10 = 0 #Initial velocity of resonator 1
    #x20 = 0.068 #Initial displacement of resonator 2
    v20 = 0
    
    DE = 20
    
    #Concatenate the parameters and prepare them for the simulation.
    params = [omega_1,gamma_1,omega_2,gamma_2,J]
    y0 = [x10,v10,x20,v20]

    time = 2*np.pi/omega_1
    pointsperosc = 100
    nperiods = 150000
    npts = pointsperosc*nperiods
    deltat = time/pointsperosc
    t = np.linspace(0.,time*nperiods,npts)

    #Perform the simulation
    psol = odeint(doubleres,y0,t,args = (params,))
    
    #Extract the two amplitudes of the resonators, and STFT them.
    px1 = pd.Series(psol[:,0])
    px2 = pd.Series(psol[:,2])

    #Use scipy.signal.stft to perform a moving window FFT. 
    fsim,tsim,Sxxsim = stft(px1,1/deltat, nperseg = 65536, return_onesided = True)
    fsim2,tsim2,Sxxsim2 = stft(px2,1/deltat, nperseg = 65536, return_onesided = True)

    Sxx2sim = np.abs(Sxxsim)
    Sxx2sim2 = np.abs(Sxxsim2)
    
    #Extract the main (dominant) frequency
    Peakidsim = int(np.argmax(Sxx2sim[0:17000,0]))
    Peakidsim2 = int(np.argmax(Sxx2sim2[0:17000,0]))

    RDfullsim = Sxx2sim[Peakidsim,:]
    RDfullsim2 = Sxx2sim2[Peakidsim2,:]
    
    #Time-dependent frequency shift, shouldn't need changing
    fdiff = 8.6*(1 - np.exp(-0.61*(tsim[2:-2]+0.64)))
    
    #Combine into total detected amplitude with interference effect from frequency difference
    sigtot = 20*np.log10(np.abs(np.exp(1j*2*np.pi*omega_1*tsim[2:-2])*RDfullsim[2:-2] + np.exp(1j*2*np.pi*(omega_1+fdiff)*tsim[2:-2])*DE*RDfullsim2[2:-2]))

    #Interpolate into data points matching measurement times (input)
    sigfin = np.interp(timedata,tsim[2:-2],sigtot)
    
    return sigfin


def Optimizecurve(timedata,valdata,p0):
    #p0 = [500,-2.5,0.08]
    pmin = [10,0,0]
    pmax = [1000,0.5,0.5]
    
    sigma = np.ones(valdata.shape[0])
    sigma[10:500] = 0.1 #(Was from 250:500 in v6)
    
    #Fit using scipy's curve_fit.
    popt, pcov = curve_fit(Simdoubleres, timedata, valdata, p0=p0, bounds = (pmin,pmax), diff_step = 1e-3, sigma = sigma)
    
    # print fit parameters
    perr = np.sqrt(np.diag(pcov))
   
    # calculate r^2 of the fit
    fitopt = Simdoubleres(timedata, *popt)
    residuals = valdata - fitopt
    ss_res = np.sum(residuals**2)
    ss_tot = np.sum((valdata-np.mean(valdata))**2)
    r_squared = 1 - (ss_res / ss_tot)
    
    fitig = Simdoubleres(timedata,*p0)
    
    return popt, pcov, perr, r_squared, fitopt, fitig

#S1 = pd.read_csv('174312/Spectrum.csv', header = 0, sep = ',')
#S2 = pd.read_csv('174312/Zoominspectrum.csv', header = 0, sep = ',')
R1 = pd.read_csv('174312/Ringdown.csv', header = 0, sep = ',')
measparam = json.load(open("174312/Measurement parameters.json"))
#%%

J = 380#680#4e5 #3e3
x10 = 0.088 #Initial displacement
x20 = 0.068

p0 = [J,x10,x20]

#Cut data to size for the optimization
timedata = R1.iloc[2270:3134,1] -10.2 
valdata = R1.iloc[2270:3134,6]+22

popt,pcov,perr,r_squared,simopt,simint = Optimizecurve(timedata,valdata,p0)
#%%
optres = {"popt":popt.tolist(),
          "pcov":pcov.tolist(),
          "perr":perr.tolist(),
          "r_squared":r_squared,
          "simpopt":simopt.tolist(),
          "simint":simint.tolist(),
          "timedata":timedata.tolist(),
          "valdata":valdata.tolist()}

with open('Optimization results 174312_v7.json', 'w') as fp:
    json.dump(optres, fp)

#%%
#fdiff = 9 - 4*np.exp(-0.45*(tsim[2:-2]))
#Jplot = J*(1-np.exp(-2.5*t[0:-1:1001]))

fig3 = plt.figure()
ax3 = fig3.add_subplot(111)

ax3.set_xlabel('Time (s)')
ax3.set_ylabel('Power (dB)')
DE = 20 #Detection efficiency of mode 2 wrt mode 1

#ax4 = ax3.twinx()
#ax4.set_ylabel('$J^2$')
#ax4.plot(t[0:-1:1001],Jplot**2, color = 'black', linestyle = 'dotted')

ax3.plot(timedata,valdata,label = 'Data', marker = '.')
#ax3.plot(tsim[2:-2],20*np.log10(RDfullsim[2:-2]), label = 'Resonator 1')
#ax3.plot(tsim[2:-2],20*np.log10(DE*RDfullsim2[2:-2]), label = 'Resonator 2')
ax3.plot(timedata,simopt, label = 'Simulated')
ax3.plot(timedata,simint, label = 'Initial', alpha = 0.5, color = 'grey')

ax3.legend()
ax3.set_xlim((0,4))

fig3.savefig("Ringing ringdown 174312 optimized_v7.png", dpi = 300, bbox_inches = 'tight')
fig3.savefig("Ringing ringdown 174312 optimized_v7.svg", format = 'svg', bbox_inches = 'tight')

