'''
Created 2019-08-28
@author Matthijs de Jong

This script is intended to explore the mechanical spectrum of the device under study in the mechanics setup.
It looks at the modes visible with and without driving, plots the spectra and returns the mode frequencies.

'''

import Tools
import time
import numpy as np
import pandas as pd
import visa
import matplotlib.pyplot as plt
import json


class Mechanicalspectrum(object):
    def __init__(self):
    
        parameters = {
        "Startfrequency":50e3,
        "Stopfrequency":550e3,
        "SAvidbandwidth":50,
        "SAbandwidth":50,
        "SGchannel":1,
        "SGnoiseamplitude":5,
        "Zoominspan":500,
        "ZoominBW":1,
        "ZoominVBW":1,
        "Ringdowndriveampl":0.2,
        "Ditherstart":0,
        "Ditherstop":0,
        "Dithersteps":1,
        "Peakampcutoff":50,
        "Ringdowndrivetime":10,
        "Ringdownwaittime":30,
        "Minpeakspace":50,
        }
        
        #If there is not a folder already, create a folder with a name consisting of the date in the folder 
        dirname,timestamp = Tools.filesetup('Mechanicalspectrum')
        
        #Connect to the spectrum analyzer
        self.SA = visa.ResourceManager().open_resource("TCPIP::192.168.1.55::inst0::INSTR")
        print("Connected to " + self.SA.query('*IDN?'))
        self.SA.timeout = 50000
        
        #Connect to the signal generator
        self.SG = visa.ResourceManager().open_resource("TCPIP::192.168.1.102::INSTR")
        print("Connected to " + self.SG.query('*IDN?'))
        
        #Turn SG output channel off if it was on
        self.SG.write(':SOUR{} OFF'.format(parameters["SGchannel"]))
        
        #Set the SA settings
        #First reset the SA
        self.SA.write("SYST:PRES")
        time.sleep(2)
        
        #Then set the SA settings
        self.SA.write("INIT:CONT OFF")
        self.SA.write("FREQ:CENT {}".format((parameters["Stopfrequency"] - parameters["Startfrequency"])/2 + parameters["Startfrequency"]))
        self.SA.write("FREQ:SPAN {}".format(parameters["Stopfrequency"] - parameters["Startfrequency"]))
        self.SA.write("BAND {}".format(parameters["SAbandwidth"]))
        self.SA.write("BAND:VID {}".format(parameters["SAvidbandwidth"]))
        self.SA.write("SWE:COUN 20")
        self.SA.write("SWE:POIN 10001")
        
        spectrum = pd.DataFrame(data = None, index = None, columns = None)
        
        self.SA.write("INIT;*WAI")
        spectrum["Freq"] = np.fromstring(self.SA.query("TRAC1:X? TRACE1"),sep = ',')
        spectrum["0 V"] = np.fromstring(self.SA.query("TRAC? TRACE1"),sep = ',')
        

        self.SG.write(":SOUR{}:APPL:NOIS {},0".format(parameters["SGchannel"],parameters["SGnoiseamplitude"]))
        self.SG.write(":OUTP{} ON".format(parameters["SGchannel"]))
        time.sleep(5)
        
        self.SA.write("INIT;*WAI")
        spectrum["{} V".format(parameters["SGnoiseamplitude"])] = np.fromstring(self.SA.query("TRAC? TRACE1"),sep = ',')

        self.SG.write(":OUTP{} OFF".format(parameters["SGchannel"]))
        
        #Convert spectra to linear range
        spec1 = np.power(10,spectrum["0 V"]/10)
        spec2 = np.power(10,spectrum["{} V".format(parameters["SGnoiseamplitude"])]/10)
        
        meanspec1 = np.mean(spec1)
        meanspec2 = np.mean(spec2)
        
        peaks1 = np.where(spec1 > parameters["Peakampcutoff"]*meanspec1)
        peaks2 = np.where(spec2 > parameters["Peakampcutoff"]*meanspec2)
        
        spectrum["Peaks1"] = spectrum["Freq"].iloc[peaks1]
        spectrum["Peaks2"] = spectrum["Freq"].iloc[peaks2]

        fig1 = plt.figure()
        ax1 = fig1.add_subplot(111)
        
        ax1.set_xlabel("Frequency (kHz)")
        ax1.set_ylabel("Power (dBm)")
        
        ax1.plot(spectrum["Freq"]/1e3,spectrum["0 V"], color = 'blue', linewidth = 0.5, label = 'No driving')
        ax1.plot(spectrum["Freq"]/1e3,spectrum["{} V".format(parameters["SGnoiseamplitude"])], color = 'red', linewidth = 0.5, label = '{} V driving'.format(parameters["SGnoiseamplitude"]))
        
        ax1.legend()
        
        fig1.savefig(dirname + "Spectrum.png", bbox_inches = 'tight', dpi = 300)

        spectrum.to_csv(path_or_buf = dirname + 'Spectrum.csv')
        
        Zoominspectrum = pd.DataFrame(data = None, index = None, columns = None)
        Ringdown = pd.DataFrame(data = None, index = None, columns = None)
        peaklist = []
        time.sleep(120)
        
        for idx,idxpeak in enumerate(peaks2[0]):
            freqpeak = spectrum["Freq"].iloc[idxpeak]
            if idx >= 1 and abs(freqpeak - peaklist[-1]) <= parameters["Minpeakspace"]:
                print("Peaks too close together, skipping. The distance was only {:.1f} Hz".format(abs(freqpeak - peaklist[-1])))
            else:
                peaklist.append(freqpeak)
                self.SA.write("FREQ:CENT {}".format(freqpeak))
                self.SA.write("FREQ:SPAN {}".format(parameters["Zoominspan"]))
                self.SA.write("BAND {}".format(parameters["ZoominBW"]))
                self.SA.write("BAND:VID {}".format(parameters["ZoominVBW"]))
                self.SA.write("SWE:COUN 1")
                self.SA.write("SWE:TIME:AUTO ON")
                self.SA.write("INIT;*WAI")
                
                Zoominspectrum["{} Freq".format(idx)] = np.fromstring(self.SA.query("TRAC1:X? TRACE1"),sep = ',')
                Zoominspectrum["{} Power".format(idx)] = np.fromstring(self.SA.query("TRAC? TRACE1"),sep = ',')

                #measurement with driving
                #print("Starting ZoomSpectrum at {:.2f} kHz".format(freqpeak/1e3))
                self.SG.write(":SOUR{}:APPL:NOIS {},0".format(parameters["SGchannel"],parameters["SGnoiseamplitude"]))
                self.SG.write(":OUTP{} ON".format(parameters["SGchannel"]))
                time.sleep(1)
            
                self.SA.write("INIT;*WAI")
                Zoominspectrum["{} PowerDriving".format(idx)] = np.fromstring(self.SA.query("TRAC? TRACE1"),sep = ',')

                self.SG.write(":OUTP{} OFF".format(parameters["SGchannel"]))   
                Zoominpeak = Zoominspectrum["{} PowerDriving".format(idx)].where((freqpeak - parameters["Minpeakspace"]) < Zoominspectrum["{} Freq".format(idx)].iloc[:]).dropna()
                Zoominpeak = Zoominpeak.where(Zoominspectrum["{} Freq".format(idx)].iloc[:] < (freqpeak + parameters["Minpeakspace"])).dropna()
                Peakfreq = Zoominspectrum["{} Freq".format(idx)].iloc[Zoominpeak.idxmax()]
                peaklist[-1] = Peakfreq
                fig2 = plt.figure()
                ax2 = fig2.add_subplot(111)
                
                ax2.set_xlabel("Frequency (kHz)")
                ax2.set_ylabel("Power (dBm)")
                
                ax2.plot(Zoominspectrum["{} Freq".format(idx)]/1e3, Zoominspectrum["{} Power".format(idx)], label = 'No driving')
                ax2.plot(Zoominspectrum["{} Freq".format(idx)]/1e3, Zoominspectrum["{} PowerDriving".format(idx)], label = '{} V driving'.format(parameters["SGnoiseamplitude"]))
                ax2.set_title("Peak at {:.3f} kHz".format(Peakfreq/1e3))
                
                ax2.legend()
                
                fig2.savefig(dirname + "Zoomin peak {}.png".format(idx), bbox_inches = 'tight', dpi = 300)
                plt.close(fig2)
                
                dithers = np.linspace(parameters["Ditherstart"],parameters["Ditherstop"],parameters["Dithersteps"])
                
                fig3 = plt.figure()
                ax3 = fig3.add_subplot(111)
            
                ax3.set_xlabel("Time (s)")
                ax3.set_ylabel("Power (dBm)")
                
                for idx2, dither in enumerate(dithers):
                    ditheredfreq = dither + Peakfreq
                    print("Starting ringdown at {} kHz".format(ditheredfreq/1e3))
                    if -0.51<dither<0.51:
                        self.SA.write("FREQ:CENT {}".format(Peakfreq))
                    elif dither<-0.51:
                        self.SA.write("FREQ:CENT {}".format(Peakfreq-1))
                    elif dither>0.51:
                        self.SA.write("FREQ:CENT {}".format(Peakfreq+1))
                    self.SA.write("FREQ:SPAN 0")
                    self.SA.write("BAND 100")
                    self.SA.write("BAND:VID 100")
                    self.SA.write("SWE:TIME {}".format(parameters["Ringdowndrivetime"]+parameters["Ringdownwaittime"]+5))
                
                    self.SG.write(":OUTP{} OFF".format(parameters["SGchannel"]))
                    self.SG.write(":SOUR{}:APPL:SIN {},{},0,0".format(parameters["SGchannel"],ditheredfreq,parameters["Ringdowndriveampl"])) 
                    self.SG.write(":OUTP{} ON".format(parameters["SGchannel"]))
                
                    time.sleep(3)
                    self.SA.write("ABOR")
                    self.SA.write("INIT")
                
                    time.sleep(parameters["Ringdowndrivetime"])
                    self.SG.write(":OUTP{} OFF".format(parameters["SGchannel"]))
                    time.sleep(parameters["Ringdownwaittime"])
                    
                    time.sleep(10)
                    Ringdown["{} Time {}".format(idx,idx2)] = np.fromstring(self.SA.query("TRAC1:X? TRACE1"),sep = ',')
                    Ringdown["{} Power {}".format(idx,idx2)] = np.fromstring(self.SA.query("TRAC? TRACE1"),sep = ',')
                    
                    ax3.plot(Ringdown["{} Time {}".format(idx,idx2)], Ringdown["{} Power {}".format(idx,idx2)], label = '{0:.2f} Hz'.format(dither))
                
                ax3.legend()
                fig3.savefig(dirname + "Ringdown {}.png".format(idx), bbox_inches = 'tight', dpi = 300)
                plt.close(fig3)
            
        Ringdown.to_csv(path_or_buf = dirname + 'Ringdown.csv')    
        Zoominspectrum.to_csv(path_or_buf = dirname + 'Zoominspectrum.csv')
         
        self.SA.close()
        self.SG.write(":OUTP{} OFF".format(parameters["SGchannel"]))
        self.SG.close()
        
        parameters["Peaklist"] = peaklist
        
        #Save the parameters in a .json file
        paramfile = open(dirname + 'Measurement parameters.json', 'w')
        json.dump(parameters,paramfile,indent = 2)
        paramfile.close()    
        
if __name__ == '__main__':
    Mechanicalspectrum()