# -*- coding: utf-8 -*-
"""
Created on Mon Oct 18 14:52:43 2021

@author: matthijsdejong
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.signal import savgol_filter
from scipy.optimize import curve_fit

def Lorentzian(x, I, x0, gamma, offset):
    #Cauchy-distribution (Lorentzian power spectrum)
    return I / ( 1 + ( ( x-x0 ) / gamma ) ** 2 ) + offset

def linearcurve(x, a, b):
    return a*x+b

def fit(f, x, y, p0):
    """ Fitting function. It uses scipy's curve_fit with bounds. It outputs the optimal 
    fitting parameters popt, the covariances pcov, the errors on the optimized parameters
    perr, and the r squared r_squared to measure the goodness of the fit. 
    """
    # fit
    popt, pcov = curve_fit(f, x, y, p0=p0)
    # sigma_fit = (y/10)
    # popt, pcov = curve_fit(f, x, y, p0=p0,  sigma=sigma_fit, absolute_sigma=True)

    # print fit parameters
    perr = np.sqrt(np.diag(pcov))
   
    # calculate r^2 of the fit
    residuals = y - f(x, *popt)
    ss_res = np.sum(residuals**2)
    ss_tot = np.sum((y-np.mean(y))**2)
    r_squared = 1 - (ss_res / ss_tot)
    
    return popt, pcov, perr, r_squared

def Calc_decoherence(timcut,datcut,fitlen):
    p0 = (-0.1,np.mean(datcut))
    popt,pcov,perr,r_squared = fit(linearcurve,timcut,datcut,p0)
    callin = linearcurve(timcut,*popt)
    fltdata = datcut - callin

    #Take autocorrelation
    Correls2 = np.correlate(fltdata,fltdata, mode = 'full')
    lendata = int(len(Correls2)/2)
    
    #Cut off only the positive time coefficients
    cut_Correls2 = Correls2[lendata-1:-1]

    #Filter and normalize to 1
    filt_Correls2 = savgol_filter(cut_Correls2,5,1)
    norm_Correls2 = filt_Correls2/np.max(filt_Correls2)

    #Fit a straight line to correlations (first part, neglecting first two points).
    p0 = (-0.1,0.6)
    popt,pcov,perr,r_squared = fit(linearcurve,Ring1.iloc[2:fitlen,0],norm_Correls2[2:fitlen],p0)
    fittimes = np.linspace(0,15,1001)
    fittedcurve = linearcurve(fittimes,*popt)
    fupper = linearcurve(fittimes,popt[0],popt[1]+perr[1])
    flower = linearcurve(fittimes,popt[0],popt[1]-perr[1])

    fig12 = plt.figure()
    ax12 = fig12.add_subplot(111)
    
    ax12.scatter(Ring1.iloc[0:P2-P1,0],norm_Correls2, s = 4)
    ax12.plot(fittimes,fittedcurve, color = 'black', linestyle = 'dashed')
    ax12.plot([0,60],[0,0], color = 'black', linestyle = 'dotted', alpha = 0.7)
    
    ax12.set_xlabel('Time delay (s)')
    ax12.set_ylabel('Autocorrelation')
    
    #Calculate the coherence time as the time where the fitted curve crosses the zero (= becomes uncorrelated)
    cohtime = fittimes[np.where(fittedcurve < 0)[0][0]]/popt[1]
    #Uncertainty of fit.
    cohunc = (fittimes[np.where(fupper < 0)[0][0]]/popt[1] - fittimes[np.where(flower < 0)[0][0]]/popt[1])/2
    ax12.set_title('Time offset {:.1f}s, Coherence time {:.2f} $\pm$ {:.3f} s'.format(timcut.iloc[0],cohtime,cohunc))
    
    fig12.savefig('Autocorrelation at {:.1f}s.png'.format(timcut.iloc[0]), dpi = 300, bbox_inches = 'tight')
    plt.close(fig12)
    return cohtime,cohunc


Spec = pd.read_csv('163500_22_outerbeamLEFT_120kHz_RBW19mHz\Spectrum.csv', header = 31, sep = ';')
Ring1 = pd.read_csv('Ringdown.csv', header = 30, sep = ';')
Ring2 = pd.read_csv('Ringdown2.csv', header = 30, sep = ';')
#%%
fig1 = plt.figure(figsize = (2.5,1.5))
ax1 = fig1.add_subplot(111)

ax1.set_xlabel('Frequency (Hz)')
ax1.set_ylabel('Noise PSD (dBm)')

R1 = 47700
R2 = 50100

ax1.plot(Spec.iloc[R1:R2,0]-120720,Spec.iloc[R1:R2,1])
ax1.set_xlim((4.8,7.15))

ax1.set_xticks((5,6,7))
ax1.set_xticklabels(('-1.0', '120726.0', '+1.0'))

fig1.savefig('Spectrum inset.png', dpi = 300, bbox_inches = 'tight')
fig1.savefig('Spectrum inset.svg', format = 'svg', bbox_inches = 'tight')

#%%
fig3 = plt.figure(figsize = (9,4))
ax3 = fig3.add_subplot(111)

ax3.set_xlabel('Time (s)')
ax3.set_ylabel('Noise PSD (dBm)')

ax3.scatter(Ring1.iloc[:,0],Ring1.iloc[:,1],label = 'Data', s = 1)
ax3.scatter(Ring2.iloc[160:-1,0]+280,Ring2.iloc[160:-1,1],label = 'Data', s = 1, color = 'Green')
ax3.set_xlim((95,580))
ax3.legend(loc = 'upper center')

fig3.savefig('Ringdown concatenated full.png', dpi = 300, bbox_inches = 'tight')
fig3.savefig('Ringdown concatenated full.svg', format = 'svg', bbox_inches = 'tight')

#%%
fig4 = plt.figure(figsize = (4,2))
ax4 = fig4.add_subplot(111)

#ax4.set_xlabel('Time (s)')
#ax4.set_ylabel('Noise PSD (dBm)')

R3 = 400
R4 = 450

ax4.plot(Ring1.iloc[R3:R4,0],Ring1.iloc[R3:R4,1], marker = '.', linewidth = 1, linestyle = 'dotted')
ax4.set_xlim((120,135))

fig4.savefig('Ringdown inset 1.png', dpi = 300, bbox_inches = 'tight')
fig4.savefig('Ringdown inset 1.svg', format = 'svg', bbox_inches = 'tight')

#%%
fig5 = plt.figure(figsize = (4,2))
ax5 = fig5.add_subplot(111)

#ax5.set_xlabel('Time (s)')
#ax5.set_ylabel('Noise PSD (dBm)')

R5 = 940
R6 = 1000
ax5.plot(Ring2.iloc[R5:R6,0]+280,Ring2.iloc[R5:R6,1], marker = '.', linewidth = 1, linestyle = 'dotted', color = 'green')
ax5.set_xlim((563,578))

fig5.savefig('Ringdown inset 2.png', dpi = 300, bbox_inches = 'tight')
fig5.savefig('Ringdown inset 2.svg', format = 'svg', bbox_inches = 'tight')

#%%

fig6 = plt.figure(figsize = (10,10))
ax6 = fig6.add_subplot(311)

ax6.set_xlabel('Time (s)')
ax6.set_ylabel('Noise PSD (dBm)')

omega_1 = 120725.555 #resonance frequency, in Hz
omega_2 = 120726.2
phi1 = 0.0011

time1 = np.linspace(400,439,1001)
simcurve1 = 10*np.log10(np.abs(np.exp(1j*2*np.pi*omega_1*time1) + np.exp(1j*2*np.pi*omega_2*(time1+phi1))))-88

phi2 = 0.0014
time2 = np.linspace(439,488.5,1001)
simcurve2 = 10*np.log10(np.abs(np.exp(1j*2*np.pi*omega_1*time2) + np.exp(1j*2*np.pi*omega_2*(time2+phi2))))-88

phi3 = 0.0018
time3 = np.linspace(488.5,540,1001)
simcurve3 = 10*np.log10(np.abs(np.exp(1j*2*np.pi*omega_1*time3) + np.exp(1j*2*np.pi*omega_2*(time3+phi3))))-88

R7 = 400
R8 = 560

ax6.plot(Ring2.iloc[R7:R8,0]+280,Ring2.iloc[R7:R8,1], marker = '.', linewidth = 1, linestyle = 'dotted', color = 'green')
ax6.plot(time1,simcurve1)
ax6.plot(time2,simcurve2)

ax6.set_xlim((400,447))
ax6.set_ylim((-110,-80))

ax9 = fig6.add_subplot(312)

ax9.set_xlabel('Time (s)')
ax9.set_ylabel('Noise PSD (dBm)')

R7 = 560
R8 = 720

ax9.plot(Ring2.iloc[R7:R8,0]+280,Ring2.iloc[R7:R8,1], marker = '.', linewidth = 1, linestyle = 'dotted', color = 'green')
#ax9.plot(time1,simcurve1)
ax9.plot(time2,simcurve2, color = 'C1')
ax9.plot(time3,simcurve3, color = 'C2')

ax9.set_xlim((447,496))
ax9.set_ylim((-110,-80))

ax10 = fig6.add_subplot(313)

ax10.set_xlabel('Time (s)')
ax10.set_ylabel('Noise PSD (dBm)')

R7 = 720
R8 = 960

ax10.plot(Ring2.iloc[R7:R8,0]+280,Ring2.iloc[R7:R8,1], marker = '.', linewidth = 1, linestyle = 'dotted', color = 'green')
ax10.plot(time3,simcurve3+3, color = 'C2')

ax10.set_xlim((496,535))
ax10.set_ylim((-110,-80))

#%%
#Shift the data towards the mean, otherwise the autocorrelation does weird things.
Davg = Ring2.iloc[400:960,1] - np.mean(Ring2.iloc[400:960,1])

#Take autocorrelation
Correls = np.correlate(Davg,Davg, mode = 'full')

#Cut it down to half its lenght (only positive time) and average out the beating pattern.
#Also normalize
lendata = int(len(Correls)/2)
cut_Correls = Correls[lendata-1:-1]
filt_Correls = savgol_filter(cut_Correls,5,1)
norm_Correls = filt_Correls/np.max(filt_Correls)

#Fit a straight line to it.
p0 = (-0.1,0.6)
popt,pcov,perr,r_squared = fit(linearcurve,Ring2.iloc[2:40,0],norm_Correls[2:40],p0)
fittimes = np.linspace(0,20,1001)
fittedcurve = linearcurve(fittimes,*popt)
fupper = linearcurve(fittimes,popt[0],popt[1]+perr[1])
flower = linearcurve(fittimes,popt[0],popt[1]-perr[1])

#Calculate the coherence time as the time where the fitted curve crosses the zero (= becomes uncorrelated)
cohtime = fittimes[np.where(fittedcurve < 0)[0][0]]

#Uncertainty of fit.
uncwind = (fittimes[np.where(fupper < 0)[0][0]] - fittimes[np.where(flower < 0)[0][0]])/2

#%%


indsteps = np.linspace(0,800,41,dtype = int)
cohs = np.zeros((41,2))
'''
for idx,ind in enumerate(indsteps):
    #Get separate part at the start of the measurement, to confirm the autocorrelation.
    P1 = int(ind)
    P2 = P1+200
    
    datcut = Ring1.iloc[P1:P2,1]
    timcut = Ring1.iloc[P1:P2,0]

    cohtime,cohunc = Calc_decoherence(timcut,datcut,20)
    cohs[idx,:] = cohtime,cohunc
    
'''
#%%    
#Ran this to store the results
#Ring1_cohs = np.savetxt('Ringdown 1 coherence.txt',cohs)
#Or load the results
Ring1_coh = np.loadtxt('Ringdown 1 coherence.txt')

#Ran this to store the results
#Ring2_cohs = np.savetxt('Ringdown 2 coherence.txt',cohs)
#Or load the results
Ring2_coh = np.loadtxt('Ringdown 2 coherence.txt')

#%%
fig13 = plt.figure()
ax13 = fig13.add_subplot(111)

ax13.errorbar(Ring1.iloc[indsteps,0],Ring1_coh[:,0],2*Ring1_coh[:,1])
ax13.errorbar(Ring2.iloc[indsteps,0]+280,Ring2_coh[:,0],2*Ring2_coh[:,1], color = 'green')

ax13.set_xlabel('Time (s)')
ax13.set_ylabel('Coherence time (s)')
ax13.set_xlim((0,530))
ax13.set_ylim((5,25))
#%%
fig11 = plt.figure(figsize = (4,2))
ax11 = fig11.add_subplot(111)

ax11.set_xlabel('Time (s)')
ax11.set_ylabel('Autocorrelation')
npts = np.linspace(1,lendata,lendata)
ax11.scatter(Ring2.iloc[0:lendata+1,0],norm_Correls)
ax11.plot(fittimes, fittedcurve, linestyle = 'dashed', color = 'black')
ax11.plot([0,40],[0,0], color = 'black', linestyle = 'dotted', alpha = 0.5)
ax11.set_xlim((0,30))
ax11.set_ylim((-0.1,1))

#fig11.savefig('Correlation of thermal noise.png', dpi = 300, bbox_inches = 'tight')
#fig11.savefig('Correlation of thermal noise.svg', format = 'svg', bbox_inches = 'tight')


#%%
fig7 = plt.figure()
ax7 = fig7.add_subplot(111)

ax7.set_xlabel('Time (s)')
ax7.set_ylabel('Noise PSD (dBm)')

R9 = 800
R10 = 860
ax7.plot(Ring2.iloc[R9:R10,0]+280,Ring2.iloc[R9:R10,1], marker = '.', linewidth = 1, linestyle = 'dotted', color = 'green')
ax7.plot(time,simcurve)

ax7.set_xlim((520,535))
ax7.set_ylim((-110,-80))

#%%
fig8 = plt.figure(figsize = (4,2))
ax8 = fig8.add_subplot(111)

ax8.set_xlabel('Frequency (Hz)')
ax8.set_ylabel('Noise PSD (dBm)')

f1 = np.linspace(120724.7,120727,1001)
Lzian1 = Lorentzian(f1,6e-10,120725.518,0.0025,1e-15)
Lzian2 = Lorentzian(f1,3.1e-9,120726.14,0.0023,1e-15)
Lziantot = Lzian1 + Lzian2
R1 = 47700
R2 = 50100

ax8.scatter(Spec.iloc[R1:R2,0]-120720,Spec.iloc[R1:R2,1], s=3)
ax8.plot(f1-120720,10*np.log10(Lziantot), color = 'C1' )

ax8.set_xlim((4.8,7))
ax8.set_xticks((5.0,6.0,7.0))
ax8.set_xticklabels(('-1.0','120726.0','+1.0'))
ax8.set_ylim((-155,-81))

#fig8.savefig('Spectrum inset.png', dpi = 300, bbox_inches = 'tight')
#fig8.savefig('Spectrum inset.svg', format = 'svg', bbox_inches = 'tight')





