# -*- coding: utf-8 -*-
"""
Created on Wed Sep  1 14:22:03 2021

@author: matthijsdejong
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.integrate import odeint
from odeintw import odeintw

#odeintw is a package from https://pypi.org/project/odeintw/
#For using odeint with complex functions.

#I'm trying to replicate the simulations from https://www-nature-com.tudelft.idm.oclc.org/articles/nnano.2017.86
#From Güttinger et al.,
#Energy-dependent path of dissipation in nanomechanical resonators
#Nature Nanotechnology 12 (2017)

#In particular, Eqs. S15 and S16 from their supplementary
def EQ15and16(avec,t,params):
    a1,a2 = avec
    Omega,omega_1,omega_2,alpha1,alpha2,gamma1,gamma2,g,f = params
    derivs = [-1j*((Omega - omega_1)*a1 - 3/(2*Omega)*alpha1*a1*np.abs(a1)**2 + 3/(2*Omega)*g*np.conj(a1)**2*a2) - gamma1/2*a1+f/(4*Omega),
              -1j*((3*Omega- omega_2)*a2 - 1/(2*Omega)*alpha2*a2*np.abs(a2)**2 + g/(6*Omega)*a1**3)-gamma2/2*a2]
    return derivs

#Also Eqs S26 and S27, which describe a n:1 coupling (I took n = 1)
def EQ26and27(avec,t,params):
    a1,a2 = avec
    delta1,delta2,alpha1,alpha2,gamma1,gamma2,g,f = params
    derivs = [-1j*(delta1*a1 - 3/2*alpha1*a1*np.abs(a1)**2 + g/2*a2) - gamma1/2*a1+f/2,
              -1j*(delta2*a2 - 3/2*alpha2*a2*np.abs(a2)**2 + g/2*a1) - gamma2/2*a2]
    return derivs

R1 = pd.read_csv('174312/Ringdown.csv', header = 0, sep = ',')

#%%
f = 0 #Drive
gamma1 = 2*np.pi*0.1#93 #Linear dissipation of mode 1
gamma2 = 2*np.pi*0.3#1.9e3 #Linear dissipation of mode 2
a01 = 3e-1+0j #Complex initial ampltiudes of modes 1,2
a02 = 8e-1+0j
g = 1e8#1e15#1.25e33#Coupling parameter between resonators.
alpha1 = 5e8#-1e18#-8.3e32 #Duffing nonlinearities of modes 1,2
alpha2 = 9e8#1e18#5.25e34

Omega = 2*np.pi*240e3 #Frequency of rotating frame (?)
omega_1 = 2*np.pi*240e3 #Unperturbed resonance frequency of resonator 1
omega_2 = 2*np.pi*720e3 #Idem, for resonator 2 (perfect 3:1 correspondence)

params = [Omega,omega_1,omega_2,alpha1,alpha2,gamma1,gamma2,g,f]
a0 = [a01,a02]

npts = 5000
deltat = 0.001
t = np.linspace(0.,npts*deltat,npts)

psol = odeintw(EQ15and16,a0,t,args = (params,))

fig1 = plt.figure()
ax1 = fig1.add_subplot(111)

ax1.set_xlabel('Time (rescaled?)')
ax1.set_ylabel('Mode amplitude (ish)')

ax1.plot(t,np.abs(psol[:,0]), label = 'Mode 1')
ax1.plot(t,np.abs(psol[:,1]), label = 'Mode 2')
ax1.set_yscale('log')
ax1.legend()

#%%
f = 0
gamma1 = 0.35
gamma2 = 3*2*2
a01 = 1e-4+0j #Complex initial ampltiudes of modes 1,2
a02 = 221e-1+0j
g = 20#1e15#1.25e33#Coupling parameter between resonators.
alpha1 = 100#5e8#-1e18#-8.3e32 #Duffing nonlinearities of modes 1,2
alpha2 = 35#9e8#1e18#5.25e34
delta1 = 4.5
delta2 = -4.5
omega_1 = 240e3
omega_2 = 240e3

params = [delta1,delta2,alpha1,alpha2,gamma1,gamma2,g,f]
a0 = [a01,a02]

npts = 10000
deltat = 0.001
t = np.linspace(0.,npts*deltat,npts)

psol = odeintw(EQ26and27,a0,t,args = (params,))

fig2 = plt.figure(figsize = (6,3))
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Noise PSD (dBm)')

ax2.plot(R1.iloc[2200:7000,5],R1.iloc[2200:7000,6], label = 'Data', marker = '.')

ax2.plot(t+10.2,10*np.log10(np.abs(psol[:,0]))-40, label = 'Theory, Mode 1', color = 'C1', linewidth = 0.5)
ax2.plot(t+10.2,10*np.log10(np.abs(psol[:,1]))-40, label = 'Theory, Mode 2', color = 'red', linewidth = 0.5)

Combo = 10*np.log10(np.abs(psol[:,0] + psol[:,1]))-40
ax2.plot(t+10.2,Combo, label = 'Theory, Combined', color = 'black', linewidth = 1, alpha = 0.7)
#ax2.set_yscale('log')
ax2.legend()
ax2.set_xlim((10.2,14))
ax2.set_ylim((-80,-20))

#fig2.savefig('Internal resonance fit.png', dpi = 300, bbox_inches = 'tight')
#fig2.savefig('Internal resonance fit.svg', format = 'svg', bbox_inches ='tight')

#%%
omega_2eff = omega_2 + 2*alpha2*np.abs(psol[:,1])**2/(8*omega_2)
omega_1eff = omega_1 + 2*alpha1*np.abs(psol[:,0])**2/(8*omega_1)

#%%
f = 0
gamma1 = 0.34*2
gamma2 = 3*2
a01 = 0.0890+0j #Complex initial ampltiudes of modes 1,2
a02 = 0.0712+0j
g = 1#1e15#1.25e33#Coupling parameter between resonators.
alpha1 = 10000#100#5e8#-1e18#-8.3e32 #Duffing nonlinearities of modes 1,2
alpha2 = 0##35#9e8#1e18#5.25e34
delta1 = 0
delta2 = 0#2*np.pi*8.6
omega_1 = 240e3
omega_2 = 240e3

params = [delta1,delta2,alpha1,alpha2,gamma1,gamma2,g,f]
a0 = [a01,a02]

npts = 4000
deltat = 0.001
t = np.linspace(0.,npts*deltat,npts)

psol = odeintw(EQ26and27,a0,t,args = (params,))

fig2 = plt.figure(figsize = (6,3))
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Noise PSD (dBm)')

DE = 20

ax2.plot(t+10.2,20*np.log10(np.abs(psol[:,0])), label = 'Theory, Mode 1', color = 'C1', linewidth = 0.5)
ax2.plot(t+10.2,20*np.log10(np.abs(DE*psol[:,1])), label = 'Theory, Mode 2', color = 'red', linewidth = 0.5)

Combo = 20*np.log10(np.abs(psol[:,0] + DE*psol[:,1]))
ax2.plot(t+10.2,Combo, label = 'Theory, Combined', color = 'black', linewidth = 1, alpha = 0.7)
#ax2.set_yscale('log')
ax2.legend()
ax2.set_xlim((10.2,14))
#ax2.set_ylim((-100,0))

#%%
#Replicate zero coupling, zero nonlinearity but non-zero detuning
f = 0
gamma1 = 0.34*2
gamma2 = 3*2
a01 = 0.0890+0j #Complex initial ampltiudes of modes 1,2
a02 = 0.0712+0j
g = 0#1e15#1.25e33#Coupling parameter between resonators.
alpha1 = 0#100#5e8#-1e18#-8.3e32 #Duffing nonlinearities of modes 1,2
alpha2 = 0##35#9e8#1e18#5.25e34
delta1 = 0
delta2 = 2*np.pi*8.6
omega_1 = 240e3
omega_2 = 240e3

params = [delta1,delta2,alpha1,alpha2,gamma1,gamma2,g,f]
a0 = [a01,a02]

npts = 4000
deltat = 0.001
t = np.linspace(0.,npts*deltat,npts)

psol = odeintw(EQ26and27,a0,t,args = (params,))

fig2 = plt.figure(figsize = (4,2))
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Power(dB)')

DE = 20

ax2.plot(t+10.2,20*np.log10(np.abs(psol[:,0])), label = 'Mode 1', color = 'C1', linewidth = 1)
ax2.plot(t+10.2,20*np.log10(np.abs(DE*psol[:,1])), label = 'Mode 2', color = 'red', linewidth = 1)

Combo = 20*np.log10(np.abs(psol[:,0] + DE*psol[:,1]))
ax2.plot(t+10.2,Combo, label = 'Combined', color = 'black', linewidth = 1, alpha = 0.7)
#ax2.set_yscale('log')
ax2.legend()
ax2.set_xlim((10.2,14))
ax2.set_ylim((-60,10))

fig2.savefig('Zero coupling, nonlinearity and detuning.png', dpi = 300, bbox_inches = 'tight')
fig2.savefig('Zero coupling, nonlinearity and detuning.svg', format = 'svg', bbox_inches = 'tight')
#%%
#Replicate zero coupling and zero detuning.
f = 0
gamma1 = 0.34*2
gamma2 = 3*2
a01 = 0.0890+0j #Complex initial ampltiudes of modes 1,2
a02 = 0.0712+0j
g = 0#1e15#1.25e33#Coupling parameter between resonators.
alpha1 = 0#100#5e8#-1e18#-8.3e32 #Duffing nonlinearities of modes 1,2
alpha2 = 0##35#9e8#1e18#5.25e34
delta1 = 0
delta2 = 0#2*np.pi*8.6
omega_1 = 240e3
omega_2 = 240e3

params = [delta1,delta2,alpha1,alpha2,gamma1,gamma2,g,f]
a0 = [a01,a02]

npts = 4000
deltat = 0.001
t = np.linspace(0.,npts*deltat,npts)

psol = odeintw(EQ26and27,a0,t,args = (params,))

fig2 = plt.figure(figsize = (4,2))
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Power(dB)')

DE = 20

ax2.plot(t+10.2,20*np.log10(np.abs(psol[:,0])), label = 'Mode 1', color = 'C1', linewidth = 0.5)
ax2.plot(t+10.2,20*np.log10(np.abs(DE*psol[:,1])), label = 'Mode 2', color = 'red', linewidth = 0.5)

Combo = 20*np.log10(np.abs(psol[:,0] + DE*psol[:,1]))
ax2.plot(t+10.2,Combo, label = 'Combined', color = 'black', linewidth = 1, alpha = 0.7)
#ax2.set_yscale('log')
#ax2.legend()
ax2.set_xlim((10.2,14))
ax2.set_ylim((-60,10))

fig2.savefig('Zero coupling, nonlinearity.png', dpi = 300, bbox_inches = 'tight')
fig2.savefig('Zero coupling, nonlinearity.svg', format = 'svg', bbox_inches = 'tight')

#%%
#Replicate strong coupling regime (no detuning, no )
f = 0
gamma1 = 0.34*2
gamma2 = 3*2
a01 = 0.0890+0j #Complex initial ampltiudes of modes 1,2
a02 = 0.0712+0j
g = 150#1e15#1.25e33#Coupling parameter between resonators.
alpha1 = 0#100#5e8#-1e18#-8.3e32 #Duffing nonlinearities of modes 1,2
alpha2 = 0##35#9e8#1e18#5.25e34
delta1 = 0
delta2 = 0
omega_1 = 240e3
omega_2 = 240e3

params = [delta1,delta2,alpha1,alpha2,gamma1,gamma2,g,f]
a0 = [a01,a02]

npts = 4000
deltat = 0.001
t = np.linspace(0.,npts*deltat,npts)

psol = odeintw(EQ26and27,a0,t,args = (params,))

fig2 = plt.figure(figsize = (4,2))
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Power(dB)')

DE = 20

ax2.plot(t+10.2,20*np.log10(np.abs(psol[:,0])), label = 'Mode 1', color = 'C1', linewidth = 0.5)
ax2.plot(t+10.2,20*np.log10(np.abs(DE*psol[:,1])), label = 'Mode 2', color = 'red', linewidth = 0.5)

Combo = 20*np.log10(np.abs(psol[:,0] + DE*psol[:,1]))
ax2.plot(t+10.2,Combo, label = 'Combined', color = 'black', linewidth = 1, alpha = 0.7)
#ax2.set_yscale('log')
#ax2.legend()
ax2.set_xlim((10.2,14))
ax2.set_ylim((-80,10))

fig2.savefig('Strong coupling, zero nonlinearity.png', dpi = 300, bbox_inches = 'tight')
fig2.savefig('Strong coupling, zero nonlinearity.svg', format = 'svg', bbox_inches = 'tight')

#%%
f = 0
gamma1 = 0.34*2
gamma2 = 3*2
a01 = 0.0890+0j #Complex initial ampltiudes of modes 1,2
a02 = 0.0712+0j
g = 150#1e15#1.25e33#Coupling parameter between resonators.
alpha1 = 0#100#5e8#-1e18#-8.3e32 #Duffing nonlinearities of modes 1,2
alpha2 = 0##35#9e8#1e18#5.25e34
delta1 = 0#-2*np.pi*8.6
delta2 = 2*np.pi*8.6
omega_1 = 240e3
omega_2 = 240e3

params = [delta1,delta2,alpha1,alpha2,gamma1,gamma2,g,f]
a0 = [a01,a02]

npts = 4000
deltat = 0.001
t = np.linspace(0.,npts*deltat,npts)

psol = odeintw(EQ26and27,a0,t,args = (params,))

fig2 = plt.figure(figsize = (4,2))
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Power(dB)')

DE = 20

ax2.plot(t+10.2,20*np.log10(np.abs(psol[:,0])), label = 'Mode 1', color = 'C1', linewidth = 0.5)
ax2.plot(t+10.2,20*np.log10(np.abs(DE*psol[:,1])), label = 'Mode 2', color = 'red', linewidth = 0.5)

Combo = 20*np.log10(np.abs(psol[:,0] + DE*psol[:,1]))
ax2.plot(t+10.2,Combo, label = 'Combined', color = 'black', linewidth = 1, alpha = 0.7)
#ax2.set_yscale('log')
#ax2.legend()
ax2.set_xlim((10.2,14))
ax2.set_ylim((-80,10))

fig2.savefig('Strong coupling, zero nonlinearity, with detuning.png', dpi = 300, bbox_inches = 'tight')
fig2.savefig('Strong coupling, zero nonlinearity, with detuning.svg', format = 'svg', bbox_inches = 'tight')

#%%
#Mode 1 strongly nonlinear
f = 0
gamma1 = 0.34*2
gamma2 = 3*2
a01 = 0.0890+0j #Complex initial ampltiudes of modes 1,2
a02 = 0.0712+0j
g = 0.5#1e15#1.25e33#Coupling parameter between resonators.
alpha1 = 10000#100#5e8#-1e18#-8.3e32 #Duffing nonlinearities of modes 1,2
alpha2 = 0##35#9e8#1e18#5.25e34
delta1 = 0
delta2 = 0#2*np.pi*8.6
omega_1 = 240e3
omega_2 = 240e3

params = [delta1,delta2,alpha1,alpha2,gamma1,gamma2,g,f]
a0 = [a01,a02]

npts = 4000
deltat = 0.001
t = np.linspace(0.,npts*deltat,npts)

psol = odeintw(EQ26and27,a0,t,args = (params,))

fig2 = plt.figure(figsize = (4,2))
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Power(dB)')

DE = 20

ax2.plot(t+10.2,20*np.log10(np.abs(psol[:,0])), label = 'Mode 1', color = 'C1', linewidth = 0.5)
ax2.plot(t+10.2,20*np.log10(np.abs(DE*psol[:,1])), label = 'Mode 2', color = 'red', linewidth = 0.5)

Combo = 20*np.log10(np.abs(psol[:,0] + DE*psol[:,1]))
ax2.plot(t+10.2,Combo, label = 'Combined', color = 'black', linewidth = 1, alpha = 0.7)
#ax2.set_yscale('log')
#ax2.legend()
ax2.set_xlim((10.2,14))
ax2.set_ylim((-60,10))

fig2.savefig('Mode 1 nonlinear, no detuning.png', dpi = 300, bbox_inches = 'tight')
fig2.savefig('Mode 1 nonlinear, no detuning.svg', format = 'svg', bbox_inches = 'tight')

#%%
#Mode 2 strongly nonlinear
f = 0
gamma1 = 0.34*2
gamma2 = 3*2
a01 = 0.0890+0j #Complex initial ampltiudes of modes 1,2
a02 = 0.0712+0j
g = 0.5#1e15#1.25e33#Coupling parameter between resonators.
alpha1 = 0#100#5e8#-1e18#-8.3e32 #Duffing nonlinearities of modes 1,2
alpha2 = 1000000##35#9e8#1e18#5.25e34
delta1 = 0
delta2 = 2*np.pi*8.6
omega_1 = 240e3
omega_2 = 240e3

params = [delta1,delta2,alpha1,alpha2,gamma1,gamma2,g,f]
a0 = [a01,a02]

npts = 4000
deltat = 0.001
t = np.linspace(0.,npts*deltat,npts)

psol = odeintw(EQ26and27,a0,t,args = (params,))

fig2 = plt.figure(figsize = (4,2))
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Power(dB)')

DE = 20

ax2.plot(t+10.2,20*np.log10(np.abs(psol[:,0])), label = 'Mode 1', color = 'C1', linewidth = 0.5)
ax2.plot(t+10.2,20*np.log10(np.abs(DE*psol[:,1])), label = 'Mode 2', color = 'red', linewidth = 0.5)

Combo = 20*np.log10(np.abs(psol[:,0] + DE*psol[:,1]))
ax2.plot(t+10.2,Combo, label = 'Combined', color = 'black', linewidth = 1, alpha = 0.7)
#ax2.set_yscale('log')
#ax2.legend()
ax2.set_xlim((10.2,14))
ax2.set_ylim((-60,10))

fig2.savefig('Mode 2 nonlinear, with detuning.png', dpi = 300, bbox_inches = 'tight')
fig2.savefig('Mode 2 nonlinear, with detuning.svg', format = 'svg', bbox_inches = 'tight')
